import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:google_map_fluttify/src/android/android.export.g.dart';
import 'package:google_map_fluttify/src/ios/ios.export.g.dart';

/// 地图标记
class Marker implements IMarker {
  Marker.android(this.androidModel);

  Marker.ios(this.iosModel, this.iosController);

  com_google_android_gms_maps_model_Marker? androidModel;
  GMSMarker? iosModel;
  GMSMapView? iosController;

  /// 获取标题
  @override
  Future<String> get title {
    return platform(
      android: (_) async => await androidModel?.getTitle() ?? '',
      ios: (_) async => await iosModel?.get_title() ?? '',
    );
  }

  /// 获取副标题
  @override
  Future<String> get snippet {
    return platform(
      android: (_) async => await androidModel?.getSnippet() ?? '',
      ios: (_) async => await iosModel?.get_snippet() ?? '',
    );
  }

  /// 获取定位信息
  Future<LatLng> get location {
    return platform(
      android: (_) async {
        final _location = await androidModel?.getPosition();
        return LatLng(
          await _location?.get_latitude() ?? 0,
          await _location?.get_longitude() ?? 0,
        );
      },
      ios: (_) async {
        final location = await iosModel?.get_position();
        return LatLng(
          await location?.latitude ?? 0,
          await location?.longitude ?? 0,
        );
      },
    );
  }

  /// 获取自定义信息
  @override
  Future<String> get object {
    return platform(
      android: (_) async {
        final data = await androidModel?.getTag();
        return data as String? ?? '';
      },
      ios: (_) async {
        final data = await iosModel?.get_userData();
        return data as String? ?? '';
      },
    );
  }

  /// 删除marker
  @override
  Future<void> remove() async {
    return platform(
      android: (_) async => androidModel?.remove(),
      ios: (_) async => iosModel?.set_map(null),
    );
  }

  /// 设置坐标
  @override
  Future<void> setCoordinate(LatLng coordinate) async {
    return platform(
      android: (_) async => androidModel?.setPosition(
        await com_google_android_gms_maps_model_LatLng.create__double__double(
          coordinate.latitude,
          coordinate.longitude,
        ),
      ),
      ios: (_) async => iosModel?.set_position(
        await CLLocationCoordinate2D.create(
          coordinate.latitude,
          coordinate.longitude,
        ),
      ),
    );
  }

  /// 设置可见性
  @override
  Future<void> setVisible(bool visible) async {
    return platform(
      android: (_) async => androidModel?.setVisible(visible),
      ios: (_) async => iosModel?.set_opacity(visible ? 1 : 0),
    );
  }

  // /// 显示弹窗
  // Future<void> showInfoWindow() async {
  //   return platform(
  //     android: (_) => androidModel.showInfoWindow(),
  //     ios: (_) => iosModel.show(iosModel, true),
  //   );
  // }

  // /// 关闭弹窗
  // Future<void> hideInfoWindow() async {
  //   return platform(
  //     android: (_) => androidModel.hideInfoWindow(),
  //     ios: (_) => iosController?.deselectAnnotation_animated(iosModel, true),
  //   );
  // }

  /// 设置图标
  @override
  Future<void> setIcon(
    ImageProvider iconProvider,
    ImageConfiguration configuration,
  ) async {
    final iconData = await iconProvider.toImageData(configuration);
    return platform(
      android: (pool) async {
        final bitmap = await android_graphics_Bitmap.create(iconData);
        final icon =
            await com_google_android_gms_maps_model_BitmapDescriptorFactory
                .fromBitmap(bitmap);
        await androidModel?.setIcon(icon);
      },
      ios: (pool) async {
        final icon = await UIImage.create(iconData);
        await iosModel?.set_icon(icon);
      },
    );
  }

  @override
  Future<LatLng> get coordinate {
    return platform(
      android: (_) async {
        final _location = await androidModel?.getPosition();
        return LatLng(
          await _location?.get_latitude() ?? 0,
          await _location?.get_longitude() ?? 0,
        );
      },
      ios: (_) async {
        final location = await iosModel?.get_position();
        return LatLng(
          await location?.latitude ?? 0,
          await location?.longitude ?? 0,
        );
      },
    );
  }

  @override
  Future<void> hideInfoWindow() {
    return platform(
      android: (_) async => androidModel?.hideInfoWindow(),
      ios: (_) async => iosController?.set_selectedMarker(null),
    );
  }

  @override
  Future<void> setAngle(double angle) {
    throw UnimplementedError();
  }

  @override
  Future<void> showInfoWindow() {
    return platform(
      android: (_) async => androidModel?.showInfoWindow(),
      ios: (_) async => iosController?.set_selectedMarker(iosModel),
    );
  }

  @override
  Future<void> startAnimation(MarkerAnimation animation) {
    throw UnimplementedError();
  }

  @override
  Future<void> setSnippet(String snippet) {
    return platform(
      android: (_) async => androidModel?.setSnippet(snippet),
      ios: (_) async => iosModel?.set_snippet(snippet),
    );
  }

  @override
  Future<void> setTitle(String title) {
    return platform(
      android: (_) async => androidModel?.setTitle(title),
      ios: (_) async => iosModel?.set_title(title),
    );
  }
}

/// 折线
class Polyline implements IPolyline {
  Polyline.android(this.androidModel);

  Polyline.ios(this.iosModel);

  com_google_android_gms_maps_model_Polyline? androidModel;
  GMSPolyline? iosModel;

  @override
  Future<void> remove() {
    return platform(
      android: (_) async => androidModel?.remove(),
      ios: (_) async => iosModel?.set_map(null),
    );
  }

  @override
  Future<void> setCoordinateList(List<LatLng> coordinateList) {
    throw UnimplementedError();
  }
}

/// 多边形
class Polygon implements IPolygon {
  Polygon.android(this.androidModel);

  Polygon.ios(this.iosModel);

  com_google_android_gms_maps_model_Polygon? androidModel;
  GMSPolygon? iosModel;

  @override
  Future<void> remove() {
    return platform(
      android: (_) async => androidModel?.remove(),
      ios: (_) async => iosModel?.set_map(null),
    );
  }

  @override
  Future<bool> contains(LatLng target) {
    throw UnimplementedError();
  }
}

/// 圆形
class Circle implements ICircle {
  Circle.android(this.androidModel);

  Circle.ios(this.iosModel);

  com_google_android_gms_maps_model_Circle? androidModel;
  GMSCircle? iosModel;

  @override
  Future<void> remove() {
    return platform(
      android: (_) async => androidModel?.remove(),
      ios: (_) async => iosModel?.set_map(null),
    );
  }

  @override
  Future<void> setCoordinate(LatLng coordinate) {
    return platform(
      android: (_) async {
        return androidModel?.setCenter(
          await com_google_android_gms_maps_model_LatLng.create__double__double(
            coordinate.latitude,
            coordinate.longitude,
          ),
        );
      },
      ios: (_) async => iosModel?.set_position(
        await CLLocationCoordinate2D.create(
          coordinate.latitude,
          coordinate.longitude,
        ),
      ),
    );
  }

  @override
  Future<void> setRadius(double radius) {
    return platform(
      android: (_) async => androidModel?.setRadius(radius),
      ios: (_) async => iosModel?.set_radius(radius),
    );
  }
}
